/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * $RCSfile: gj_gtk_new_account.c,v $
 *
 * $Author: mr $
 * $Date: 2004/01/02 12:19:57 $
 *
 * $Revision: 1.3 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "gj_main.h"
#include "gj_stream.h"
#include "gj_iq.h"
#include "gj_iq_requests.h"
#include "gj_parser.h"
#include "gj_me.h"
#include "gj_config.h"

#include "gj_gtk_new_account.h"
#include "gj_gtk_support.h"
#include "gj_gtk_connection_settings.h"
#include "gj_gtk_progress.h"


typedef struct 
{
  /* search */
  GtkWidget *window;

  GtkWidget *button_ok;
  GtkWidget *button_cancel;

  GtkWidget *combo_server;
  GtkWidget *combo_entry_server;
  GtkWidget *spinbutton_port;
  GtkWidget *entry_username;  
  GtkWidget *entry_password1;
  GtkWidget *entry_password2;

  /*
   * members
   */

  GjStream stream;
  GjProgressDialog *progress_dialog;

} GjNewAccountDialog;


static gboolean gj_gtk_na_check(GjNewAccountDialog *dialog);
static gboolean gj_gtk_na_save_settings(GjNewAccountDialog *dialog);

static gboolean gj_gtk_na_connection(GjNewAccountDialog *dialog);

static void gj_gtk_na_connected_cb(GjStream stm, gint requestID);
static void gj_gtk_na_registered_cb(GjInfoquery iq, GjInfoquery iq_related);
static void gj_gtk_na_progress_cb(gboolean cancelled, GjNewAccountDialog *dialog);

static void on_entry_password_changed(GtkEditable *editable, GjNewAccountDialog *dialog);

static void on_destroy(GtkWidget *widget, GjNewAccountDialog *dialog);
static void on_response(GtkDialog *widget, gint response, GjNewAccountDialog *dialog);


static GjNewAccountDialog *current_dialog = NULL;


gboolean gj_gtk_na_load()
{
  GjNewAccountDialog *dialog = NULL;
  GladeXML *xml = NULL;

  GList *servers = NULL;
  const gchar **server_listing = NULL;
  const gchar *str = NULL;
  gint i = 0;

  if(current_dialog)
    {
      gtk_window_present(GTK_WINDOW(current_dialog->window));
      return TRUE;
    }

  dialog = g_new0(GjNewAccountDialog, 1);

  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "new_account",
			      NULL,
			      "new_account", &dialog->window,
			      "combo_server", &dialog->combo_server,
			      "combo-entry_server", &dialog->combo_entry_server,
			      "spinbutton_port", &dialog->spinbutton_port,
			      "entry_username", &dialog->entry_username,
			      "entry_password1", &dialog->entry_password1,
			      "entry_password2", &dialog->entry_password2,
			      "button_ok", &dialog->button_ok,
			      NULL);

  gj_gtk_glade_connect(xml,
		       dialog,
		       "new_account", "response", on_response,
		       "new_account", "destroy", on_destroy,
		       "entry_password1", "changed", on_entry_password_changed,
		       "entry_password2", "changed", on_entry_password_changed,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);

  gdk_window_set_decorations (GDK_WINDOW(dialog->window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE);

  current_dialog = dialog;
  
  server_listing = gnome_jabber_get_server_list();
  
  /* set a limit of a 1000 */
  while((str = server_listing[i++]) != NULL && i < 1000)
      servers = g_list_append(servers, (gchar*)str);

  gtk_combo_set_popdown_strings(GTK_COMBO(dialog->combo_server), servers);
  
  /* set default */
  gtk_entry_set_text(GTK_ENTRY(dialog->combo_entry_server), "jabber.org");

  /* clean up */
  if(servers) 
    g_list_free(servers);

  return TRUE;
}

static gboolean gj_gtk_na_check(GjNewAccountDialog *dialog)
{
  G_CONST_RETURN gchar *username = NULL;
  G_CONST_RETURN gchar *server = NULL;

  G_CONST_RETURN gchar *password1 = NULL;
  G_CONST_RETURN gchar *password2 = NULL;

  if(!dialog)
    return FALSE;

  server = gtk_entry_get_text(GTK_ENTRY(dialog->combo_entry_server));
  username = gtk_entry_get_text(GTK_ENTRY(dialog->entry_username));

  password1 = gtk_entry_get_text(GTK_ENTRY(dialog->entry_password1));
  password2 = gtk_entry_get_text(GTK_ENTRY(dialog->entry_password2));

  if(!server || g_utf8_strlen(server, -1) < 1)
    {
      gj_gtk_show_error(_("Details Incomplete."),
			1,
			_("Server was not specified."));

      return FALSE;
    }

  if(!username || g_utf8_strlen(username, -1) < 1)
    {
      gj_gtk_show_error(_("Details Incomplete."),
			1,
			_("Username was not specified."));

      return FALSE;
    }

  if(!password1 || !password2 || 
     g_utf8_strlen(password1, -1) < 1 || 
     g_utf8_strlen(password2, -1) < 1)
    {
      gj_gtk_show_error(_("Details Incomplete."),
			1,
			_("Password was not specified."));

      return FALSE;
    }

  if(strcmp(password1, password2) != 0)
    {
      gj_gtk_show_error(_("Details Incomplete."),
			1,
			_("Passwords do NOT match."));

      return FALSE;
    }

  return TRUE;
}

static gboolean gj_gtk_na_save_settings(GjNewAccountDialog *dialog)
{
  G_CONST_RETURN gchar *server = NULL;
  gint16 port = 0;
  G_CONST_RETURN gchar *username = NULL;
  G_CONST_RETURN gchar *password = NULL;

  /* if checkbutton is selected, change the connection settings */
  server = gtk_entry_get_text(GTK_ENTRY(dialog->combo_entry_server));
  port = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(dialog->spinbutton_port));
  username = gtk_entry_get_text(GTK_ENTRY(dialog->entry_username));
  password = gtk_entry_get_text(GTK_ENTRY(dialog->entry_password1));

  /* set */
  gj_config_set_cs_server(server);
  gj_config_set_cs_port(port);
  gj_config_set_cs_username(username);
  gj_config_set_cs_password(password);

  /* save */
  gj_config_write();

  return TRUE;
}

static gboolean gj_gtk_na_connection(GjNewAccountDialog *dialog)
{
  G_CONST_RETURN gchar *server = NULL;
  G_CONST_RETURN gchar *username = NULL;
  gint16 port = 0;

  if(!dialog)
    return FALSE;

  server = gtk_entry_get_text(GTK_ENTRY(dialog->combo_entry_server));
  username = gtk_entry_get_text(GTK_ENTRY(dialog->entry_username));
  port = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(dialog->spinbutton_port));

  dialog->stream = gj_stream_new(server);

  /* set struct info */
  gj_stream_set_port(dialog->stream, port);
  gj_stream_set_connected_cb(dialog->stream, gj_gtk_na_connected_cb);

  gj_stream_build(dialog->stream);

  /* set controls state */
  dialog->progress_dialog = gj_gtk_progress_load(TRUE,
						 GTK_WINDOW(dialog->window),
						 (GjProgressDialogCallback)gj_gtk_na_progress_cb, 
						 dialog, 
						 _("Registering New Account"), 
						 _("Username is \"%s\""), 
						 username); 
  

  gtk_widget_hide(dialog->window);

  /* start */
  gj_stream_open(dialog->stream);

  return TRUE;
}

static void gj_gtk_na_connected_cb(GjStream stm, gint requestID)
{
  GjNewAccountDialog *dialog = NULL;
  G_CONST_RETURN gchar *server = NULL;
  G_CONST_RETURN gchar *username = NULL;
  G_CONST_RETURN gchar *password = NULL;
  guint port = 5222;

  dialog = current_dialog;
  if(!dialog)
    return;

  /* get information */
  server = gtk_entry_get_text(GTK_ENTRY(dialog->combo_entry_server));
  port = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(dialog->spinbutton_port));
  username = gtk_entry_get_text(GTK_ENTRY(dialog->entry_username));
  password = gtk_entry_get_text(GTK_ENTRY(dialog->entry_password1));

  /* set information */
  gj_me_set_connected_host(server);
  gj_me_set_connected_port(port);
  gj_me_set_connected_jid(username);

  /* do request */
  gj_iq_request_register_user(server,
			      username,
			      password,
			      gj_gtk_na_registered_cb,
			      NULL);
}

static void gj_gtk_na_registered_cb(GjInfoquery iq, GjInfoquery iq_related)
{
  GjNewAccountDialog *dialog = NULL;
  xmlChar *type = NULL;
  xmlNodePtr node = NULL;

  dialog = current_dialog;
  if(!dialog)
    return;

  if(dialog->progress_dialog)
    {
      gj_gtk_progress_unload(dialog->progress_dialog);
      dialog->progress_dialog = NULL;
    }

  if(!iq)
    return;

  /* check for errors? */
  node = gj_iq_get_xml(iq);
  type = gj_parser_find_by_attr_and_ret(node, (guchar*)"type");

  if(type != NULL && xmlStrcmp(type, (guchar*)"error") == 0)
    {
      xmlChar *error_code = NULL;
      xmlChar *error_message = NULL;

      g_warning("%s: registration failed", __FUNCTION__);

      if((node = gj_parser_find_by_node(node, (guchar*)"error")))
	{
	  error_code = gj_parser_find_by_attr_and_ret(node, (guchar*)"code");
	  error_message = gj_parser_find_by_node_and_ret(node, (guchar*)"error");

	  gj_gtk_show_error(_("Failed to Register New Account."),
			    atoi((gchar*)error_code),
			    (gchar*)error_message);
	}
      else
	{
	  gj_gtk_show_error(_("Failed to Register New Account."),
			    1,
			    _("Unknown Error"));
	}
      
      /* find stream and close */
      if(dialog->stream != NULL)
	{
	  gj_stream_free(dialog->stream);
	  dialog->stream = NULL;
	}

      /* re-show window */
      gtk_widget_show(dialog->window);

      /* if err 409 - user n/a, select user entry */
      if(error_code && atoi((gchar*)error_code) == 409)
	gtk_widget_grab_focus(dialog->entry_username);
      
      return;
    }

  /* ok */
  gj_gtk_show_success(_("Registration Success."));

  if(dialog->stream != NULL)
    {
      /* find stream and close */
      gj_stream_free(dialog->stream);
      dialog->stream = NULL;
    }

  /* set connection settings */
  gj_gtk_na_save_settings(dialog);

  /* close window */
  gtk_widget_destroy(dialog->window);
}

static void gj_gtk_na_progress_cb(gboolean cancelled, GjNewAccountDialog *dialog)
{
  if(!dialog)
    return;

  if(cancelled == TRUE)
    {
      gtk_widget_show(dialog->window);

      /* reset p */
      dialog->progress_dialog = NULL;
    }
}

/*
 * actual gui functions
 */
static void on_entry_password_changed(GtkEditable *editable, GjNewAccountDialog *dialog)
{
  gboolean match = FALSE;

  G_CONST_RETURN gchar *password1 = NULL;
  G_CONST_RETURN gchar *password2 = NULL;

  if(!dialog)
    return;

  password1 = gtk_entry_get_text(GTK_ENTRY(dialog->entry_password1));
  password2 = gtk_entry_get_text(GTK_ENTRY(dialog->entry_password2));

  if(password1 != NULL && g_utf8_strlen(password1, -1) > 0 && 
     password2 != NULL && g_utf8_strlen(password2, -1) > 0)
    {
      if(strcmp(password1, password2) == 0)
	match = TRUE;
    }
  else
    {
      gtk_widget_set_sensitive(GTK_WIDGET(dialog->button_ok), FALSE);
      return;
    }
  
  /* change gui based on match */
  gtk_widget_set_sensitive(GTK_WIDGET(dialog->button_ok), match);
}

static void on_destroy(GtkWidget *widget, GjNewAccountDialog *dialog)
{
  current_dialog = NULL;
  if(!dialog)
    return;

  g_free(dialog);
}

static void on_response(GtkDialog *widget, gint response, GjNewAccountDialog *dialog)
{
  if(!dialog)
    return;

  if(response == GTK_RESPONSE_OK)
    {
      if(!gj_gtk_na_check(dialog))
	return;
      
      gj_gtk_na_connection(dialog);
      return;
    }

  gtk_widget_destroy(dialog->window);
}


#ifdef __cplusplus
}
#endif
